@@ -3,10 +3,13 @@ UP42 authentication mechanism and base requests functionality
 """
 import json
 from pathlib import Path
-from typing import Dict, List, Optional, Union
+from typing import Dict, Optional, Union
 
 import requests
 import requests.exceptions
+from requests.auth import HTTPBasicAuth
+from requests_oauthlib import OAuth2Session
+from oauthlib.oauth2 import BackendApplicationClient, MissingTokenError
 from tenacity import (
     Retrying,
     wait_fixed,
@@ -24,9 +27,9 @@ logger = get_logger(__name__)
 
 class retry_if_429_error(retry_if_exception):
     """
+    Altered tenacity retry strategy that retries if the exception is an ``HTTPError``
+    with a 429 status code (too many requests).
     Adapted from https://github.com/alexwlchan/handling-http-429-with-tenacity
-    Retry strategy that retries if the exception is an ``HTTPError`` with
-    a 429 status code.
     """
 
     def __init__(self):
@@ -42,9 +45,9 @@ class retry_if_429_error(retry_if_exception):
 class Auth:
     def __init__(
         self,
-        cfg_file: Union[str, Path] = None,
-        project_id: str = None,
-        project_api_key: str = None,
+        cfg_file: Union[str, Path, None] = None,
+        project_id: Optional[str] = None,
+        project_api_key: Optional[str] = None,
         **kwargs,
     ):
         """
@@ -64,6 +67,7 @@ class Auth:
         self.project_id = project_id
         self.project_api_key = project_api_key
         self.workspace_id: Optional[str] = None
+        self.token: Optional[str] = None
 
         try:
             self.env: str = kwargs["env"]
@@ -77,10 +81,6 @@ class Auth:
             self.retry: bool = kwargs["retry"]
         except KeyError:
             self.retry = True
-        try:
-            self.get_info: bool = kwargs["get_info"]
-        except KeyError:
-            self.get_info = True
 
         if self.authenticate:
             self._find_credentials()
@@ -131,31 +131,23 @@ class Auth:
         """Gets the endpoint."""
         return f"https://api.up42.{self.env}"
 
-    # pylint: disable=assignment-from-no-return
     def _get_token(self):
+        """Project specific authentication via project id and project api key."""
         try:
-            self._get_token_project()
-        except requests.exceptions.HTTPError as err:
+            client = BackendApplicationClient(
+                client_id=self.project_id, client_secret=self.project_api_key
+            )
+            auth = HTTPBasicAuth(self.project_id, self.project_api_key)
+            get_token_session = OAuth2Session(client=client)
+            token_response = get_token_session.fetch_token(
+                token_url=self._endpoint() + "/oauth/token", auth=auth
+            )
+        except MissingTokenError as err:
             raise ValueError(
                 "Authentication was not successful, check the provided project credentials."
             ) from err
 
-    def _get_token_project(self) -> None:
-        """Project specific authentication via project id and project api key."""
-        url = (
-            f"https://{self.project_id}:{self.project_api_key}@api.up42.{self.env}"
-            f"/oauth/token"
-        )
-        payload = "grant_type=client_credentials"
-        headers = {
-            "Content-Type": "application/x-www-form-urlencoded",
-            "cache-control": "no-cache",
-        }
-        token_response = requests.request("POST", url, data=payload, headers=headers)
-        token_response.raise_for_status()
-        token = json.loads(token_response.text)
-        # pylint: disable=attribute-defined-outside-init
-        self.token = token["data"]["accessToken"]
+        self.token = token_response["data"]["accessToken"]
 
     def _get_workspace(self) -> None:
         """Get workspace id belonging to authenticated project."""
@@ -186,7 +178,7 @@ class Auth:
         reraise=True,
     )
     def _request_helper(
-        self, request_type: str, url: str, data: Dict = {}, querystring: Dict = {}
+        self, request_type: str, url: str, data: dict = {}, querystring: dict = {}
     ) -> requests.Response:
         """
         Helper function for the request, running the actual request with the correct headers.
@@ -200,7 +192,7 @@ class Auth:
         Returns:
             The request response.
         """
-        headers = self._generate_headers(self.token)
+        headers = self._generate_headers(self.token)  # type: ignore
         if querystring == {}:
             response = requests.request(
                 method=request_type, url=url, data=json.dumps(data), headers=headers
@@ -222,15 +214,18 @@ class Auth:
         self,
         request_type: str,
         url: str,
-        data: Union[Dict, List] = {},
-        querystring: Dict = {},
+        data: Union[dict, list] = {},
+        querystring: dict = {},
         return_text: bool = True,
-    ) -> Union[str, Dict, requests.Response]:
+    ):  # Union[str, dict, requests.Response]:
         """
-        Handles retrying the request and automatically gets a new token if the old
-        is invalid.
+        Handles retrying the request and automatically retries and gets a new token if
+        the old is invalid.
 
-        Retry is enabled by default, can be set to False as kwargs in Api().
+        Retry is enabled by default, can be set to False as kwargs of Auth.
+
+        In addition to this retry mechanic, 429-errors (too many requests) are retried
+        more extensively in _request_helper.
 
         Args:
             request_type: 'GET', 'POST', 'PUT', 'PATCH', 'DELETE'
@@ -247,8 +242,8 @@ class Auth:
         try:
             if self.retry:
                 retryer = Retrying(
-                    stop=stop_after_attempt(1),  # TODO: Find optimal retry solution
-                    wait=wait_fixed(0),
+                    stop=stop_after_attempt(2),
+                    wait=wait_fixed(0.5),
                     retry=(
                         retry_if_exception_type(requests.exceptions.HTTPError)
                         | retry_if_exception_type(requests.exceptions.ConnectionError)
@@ -276,12 +271,10 @@ class Auth:
                 response_text = response.text
 
             # Handle api error messages here before handling it in every single function.
-            # pylint: disable=no-else-raise
             try:
                 if response_text["error"] is not None and response_text["data"] is None:
                     raise ValueError(response_text["error"])
-                else:
-                    return response_text
+                return response_text
             except (
                 KeyError,
                 TypeError,
